﻿/*
This source file is part of the RGBZ Video Camera project published in the paper

  Coherent Spatiotemporal Filtering, Upsampling and Rendering of RGBZ Videos
  Christian Richardt, Carsten Stoll, Neil A. Dodgson, Hans-Peter Seidel, Christian Theobalt
  Computer Graphics Forum (Proceedings of Eurographics), 31(2), May 2012
  <http://www.mpi-inf.mpg.de/resources/rgbz-camera/>

If you use our code, please cite our paper. Thanks.

Copyright (c) 2012 Christian Richardt <christian@richardt.name>


Code licence: GPL v3
----------------------------------------------------------------------------------------------------
This program is free software; you can redistribute it and/or modify it under the terms
of the GNU General Public License as published by the Free Software Foundation;
either version 3 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program;
if not, see <http://www.gnu.org/licenses/>.

$Id: FillInUpsamplingFilter.frag 537 2012-04-04 15:35:42Z cr333 $
*/
//#version 130
#extension GL_EXT_gpu_shader4 : enable

// input texture
uniform sampler2D lores;
uniform sampler2D hires;

// parameters
uniform int factor;
uniform int sampleOffset;

void main()
{
	ivec2 loResTexel = (ivec2(gl_FragCoord.xy) - ivec2(sampleOffset)) / factor;
	ivec2 hiResTexel = ivec2(gl_FragCoord.xy);

	// read pixel values (equivalent to texelFetch(..., ivec2(texel), 0))
	vec4 loResPixel = texture2D(lores, (vec2(loResTexel) + 0.5) / (vec2(textureSize2D(lores, 0))));
	vec4 hiResPixel = texture2D(hires, gl_FragCoord.xy / (vec2(textureSize2D(hires, 0))));
	
	// per default, use existing hi-res value
	gl_FragColor = hiResPixel;
	
	//if(hiResPixel.w == 0.0) gl_FragColor = vec4(0.0);
	
	// use new samples only in invalid areas
	if(hiResPixel.w == 0.0 && loResPixel.w != 0.0)
	if(hiResTexel.x == factor * loResTexel.x + sampleOffset)
	if(hiResTexel.y == factor * loResTexel.y + sampleOffset)
		gl_FragColor = loResPixel;

	//if(hiResPixel.w == 0.0) gl_FragColor += 0.2;
}