﻿/*
This source file is part of the RGBZ Video Camera project published in the paper

  Coherent Spatiotemporal Filtering, Upsampling and Rendering of RGBZ Videos
  Christian Richardt, Carsten Stoll, Neil A. Dodgson, Hans-Peter Seidel, Christian Theobalt
  Computer Graphics Forum (Proceedings of Eurographics), 31(2), May 2012
  <http://www.mpi-inf.mpg.de/resources/rgbz-camera/>

If you use our code, please cite our paper. Thanks.

Copyright (c) 2012 Christian Richardt <christian@richardt.name>


Code licence: GPL v3
----------------------------------------------------------------------------------------------------
This program is free software; you can redistribute it and/or modify it under the terms
of the GNU General Public License as published by the Free Software Foundation;
either version 3 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program;
if not, see <http://www.gnu.org/licenses/>.

$Id: RecorderForm.cs 537 2012-04-04 15:35:42Z cr333 $
*/
using System;
using System.IO;
using System.Threading;
using System.Windows.Forms;
using RGBZcamera.Library.Config;

namespace RGBZcamera.Recorder
{
	public partial class RecorderForm : Form
	{
		protected Settings settings = new Settings();

		public RecorderForm()
		{
			InitializeComponent();

			// Find directory to store the captured data.
			var dataPath = ConfigReader.FindDataDirectory(".");
			if (dataPath != null)
				settings.CapturePath = Path.Combine(dataPath.FullName, "captured");

			propertyGrid.SelectedObject = settings;
		}

		private void previewButton_Click(object sender, EventArgs e)
		{
			previewButton.Enabled = false;
			recordButton.Enabled  = false;
			stopButton.Enabled    = true;

			new Thread(delegate() { StartVideo(false); }).Start();
		}

		private void recordButton_Click(object sender, EventArgs e)
		{
			previewButton.Enabled = false;
			recordButton.Enabled  = false;
			stopButton.Enabled    = true;

			new Thread(delegate() { StartVideo(true); }).Start();
		}

		private void stopButton_Click(object sender, EventArgs e)
		{
			stopButton.Enabled = false;

			new Thread(delegate() { StopVideo(); }).Start();
		}

		private void StartVideo(bool record)
		{
			try
			{
				RecorderProgram.Start(this, settings, record);
			}
			catch (Exception ex)
			{
				MessageBox.Show(
					String.Format("{0}: {1}\n\n{2}\n\n(DataDisplay.StartVideo)", ex.GetType().Name, ex.Message, ex),
					String.Format("Error: {0}", ex.GetType()), MessageBoxButtons.OK, MessageBoxIcon.Error);
				Application.Exit();
				return;
			}
		}

		private void StopVideo()
		{
			// first stop the video
			RecorderProgram.Stop(this);

			// then update the buttons accordingly
			this.BeginInvoke(new Action(delegate()
			{
				previewButton.Enabled = true;
				recordButton.Enabled  = true;
			}));
		}
	}
}
