// LibAvWrap -- .NET wrapper for the libav* libraries
// --------------------------------------------------
// By Christian Richardt <christian@richardt.name>
// (based on AvDn 0.7, www.informikon.com/tutorials/)
//
// $Id: Stream.h 436 2010-10-15 16:26:38Z cr333 $

#pragma once

using namespace System;
using namespace System::Text;
using namespace System::Runtime::InteropServices;

extern "C" {
#include "libavformat/avformat.h"
}

#include "Wrapper.h"

///<summary>Wrapper for some lavc and lavf structures.</summary>
namespace LibAvWrap
{
	// forward declaration(s)
	ref class LibAvCodecContext;

	///<summary>Wrapper for the AVStream structure.</summary>
	public ref class LibAvStream : Wrapper<AVStream>
	{
// constructors and destructor --------------------------------------------------------------------

	internal:
		///<summary>Construct from a given native pointer.</summary>
		LibAvStream(AVStream* p) : Wrapper<AVStream>(p) { };
	
	public:
		///<summary>Construct from a given native address.</summary>
		LibAvStream(IntPtr ptr) : Wrapper<AVStream>(ptr) { };

		///<summary>Default destructor.</summary>
		~LibAvStream() { }

// member functions -------------------------------------------------------------------------------

// properties -------------------------------------------------------------------------------------

	public:
		///<summary>The codec context associated with this stream.</summary>
		property LibAvCodecContext^ Codec { LibAvCodecContext^ get(); }

		///<summary>Stream index in AVFormatContext.</summary>
		property int Index { int get(); }

		///<summary>Property for AVStream pts val field</summary>
		property Int64 PtsVal { Int64 get(); void set(Int64 value); }

		///<summary>Numerator of fundamental unit of time (in seconds) in terms of which frame timestamps are represented.</summary>
		property int TimeBaseNum { int get(); void set(int value); }
		
		///<summary>Denominator of fundamental unit of time (in seconds) in terms of which frame timestamps are represented.</summary>
		property int TimeBaseDen { int get(); void set(int value); }
	};

} // namespace LibAvWrap