﻿// $Id: LinearTransform.cs 535 2011-05-11 21:37:32Z cr333 $
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.ComponentModel;
using OpenTK;

namespace VideoLib.OpenGL
{
	/// <summary>Applies a linear transform to each pixel.</summary>
	public class LinearTransform : FboNode
	{
		/// <summary>Gets or sets the scale factors (result = factor * input + offset).</summary>
		[Uniform, Description("Scale factors to apply to values (result = factor * input + offset).")]
		public Vector4 Factor { get; set; }

		/// <summary>Gets or sets the offsets (result = factor * input + offset).</summary>
		[Uniform, Description("Offsets to apply to values (result = factor * input + offset).")]
		public Vector4 Offset { get; set; }

		/// <summary>Initialises a new instance of the <see cref="LinearTransform"/> class.</summary>
		public LinearTransform()
		{
			Factor = new Vector4(1, 1, 1, 1);
			Offset = new Vector4(0, 0, 0, 0);

			AddDefaultPins();
		}

		/// <summary>Initialises the OpenGL state.</summary>
		public override void Init()
		{
			// load, compile and link shaders
			programID = Utils.LinkShaders(
				Utils.CreateShaderFromResource("Shaders.LinearTransform.frag")
			);

			// map textures
			textureMap = new List<TextureBinding> {
				new TextureBinding { Pin = "in", Uniform = "input", Unit = 0 },
			};

			base.Init();
		}

		/// <summary>Draws to the framebuffer object using the given input textures.</summary>
		/// <param name="inputs">The input textures.</param>
		/// <returns>The resulting texture.</returns>
		public override TextureFrame Draw(Dictionary<string, TextureFrame> inputs)
		{
			// allocate a texture that is identical to the input
			var resultTexture = TextureFrame.CreateToMatch(inputs["in"]);
			resultTexture.Alloc();

			Draw(inputs, resultTexture.Texture);

			return resultTexture;
		}
	}
}
