﻿// $Id: FlowMorpher.cs 494 2010-11-15 20:31:09Z cr333 $
using System.Collections.Generic;
using System.ComponentModel;

namespace VideoLib.OpenGL.OpticalFlow
{
	/// <summary>Morphs between textures using computed optical flow.</summary>
	public class FlowMorpher : FboNode
	{
		/// <summary>Initialises a new instance of the <see cref="FlowMorpher"/> class.</summary>
		public FlowMorpher()
		{
			Value = 0.5f;

			AddInputPins("source", "target", "flow");
			AddOutputPin("out");
		}

		/// <summary>Gets or sets the value to morph between source (0) and target (1) textures.</summary>
		[Uniform(), Description("Value to morph between source (0) and target (1) textures.")]
		public float Value { get; set; }

		/// <summary>Initialises the OpenGL state.</summary>
		public override void Init()
		{
			// load, compile and link shaders
			programID = Utils.LinkShaders(
				Utils.CreateShaderFromResource("OpticalFlow.Shaders.FlowMorpher.frag")
			);

			// map textures
			textureMap = new List<TextureBinding> {
				new TextureBinding { Name = "source", Unit = 0 },
				new TextureBinding { Name = "target", Unit = 1 },
				new TextureBinding { Name = "flow",   Unit = 2 },
			};

			base.Init();
		}

		/// <summary>Carries out drawing to the framebuffer object using the given input textures.</summary>
		/// <param name="inputs">The input textures.</param>
		/// <returns>The resulting texture.</returns>
		public override TextureFrame Draw(Dictionary<string,TextureFrame> inputs)
		{
			// allocate a texture that is identical to the input
			var resultTexture = TextureFrame.CreateToMatch(inputs["source"]);
			resultTexture.Alloc();

			Draw(inputs, resultTexture.Texture);

			return resultTexture;
		}
	}
}
