/*
This source code is part of the Floating Textures test program, which
is a derivative of the complete Floating Textures program containing
the offline version with prereconstructed geometry and prerecorded
video. 
For more information on Floating Textures please have a look at the article

"Floating Textures" 
by M. Eisemann, B. De Decker, M. Magnor, 
P. Bekaert, E. de Aguiar, N. Ahmed, C. Theobalt, A. Sellent
Computer Graphics Forum (Proc. Eurographics EG'08), vol. 27, no. 2, April 2008 


Copyright (C) 2008 Martin Eisemann

This program is free software; you can redistribute it and/or modify it under 
the terms of the GNU General Public License as published by the Free Software 
Foundation; either version 3 of the License, or (at your option) any later 
version.

This program is distributed in the hope that it will be useful, but WITHOUT 
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS 
FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with 
this program; if not, see <http://www.gnu.org/licenses/>.
*/

/**
 * \brief Phi3.frag
 *
 * \brief Part of the optimization. Calculates the phi
 *
 */ 

uniform vec2 dxy;

uniform sampler2D texUVField;
uniform sampler2D texDUVField;

const vec2 epsilon = vec2(0.001);

void main()
{
  vec2 texCoord = gl_TexCoord[0].xy;
  vec2 tCyxm1 = texCoord + vec2(-dxy.x, 0.0);
  vec2 tCyxp1 = texCoord + vec2(dxy.x, 0.0);
  vec2 tCym1x = texCoord + vec2(0.0, -dxy.y);
  vec2 tCyp1x = texCoord + vec2(0.0, dxy.y);

  vec4 uvyxp1  = texture2D(texUVField, tCyxp1);
  vec4 duvyxp1 = texture2D(texDUVField, tCyxp1);
  vec4 uvyxm1  = texture2D(texUVField, tCyxm1);
  vec4 duvyxm1 = texture2D(texDUVField, tCyxm1);

  vec4 first = uvyxp1 + duvyxp1 - uvyxm1 - duvyxm1;
  first = first * first;


  vec4 uvyp1x  = texture2D(texUVField, tCyp1x);
  vec4 duvyp1x = texture2D(texDUVField, tCyp1x);
  vec4 uvym1x  = texture2D(texUVField, tCym1x);
  vec4 duvym1x = texture2D(texDUVField, tCym1x);

  vec4 second =  uvyp1x + duvyp1x - uvym1x - duvym1x;
  second = second * second;


  vec2 p = 0.25 * (first.xz + first.yw + second.xz + second.yw);
  p = 0.5 / sqrt( p + epsilon );

  gl_FragColor = vec4(p.x, p.x, p.y, p.y); 
}
