﻿// $Id: FrameRateCounter.cs 436 2010-10-15 16:26:38Z cr333 $
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace VideoLib.Utils
{
	/// <summary>
	/// Class that computes a frames-per-second figure based on incoming frames.
	/// </summary>
	public class FrameRateCounter
	{
		/// <summary>Gets the current frames-per-second figure.</summary>
		/// <value>The FPS.</value>
		public double FPS { get; protected set; }

		/// <summary>The averaging period, in seconds.</summary>
		protected double averagingPeriod = 1.0;

		/// <summary>The weight given to the last averaging period (exponential averaging).</summary>
		protected double averagingWeight = 0.5;

		/// <summary>Start of the most recent averaging period.</summary>
		protected DateTime lastTime = DateTime.Now;

		/// <summary>Number of frames received in the most recent averaging period.</summary>
		protected int frameCounter = 0;

		/// <summary>Initializes a new instance of the <see cref="FrameRateCounter"/> class.</summary>
		public FrameRateCounter()
		{
		}

		/// <summary>Initializes a new instance of the <see cref="FrameRateCounter"/> class.</summary>
		/// <param name="averagingPeriod">The averaging period.</param>
		/// <param name="averagingWeight">The averaging weight.</param>
		public FrameRateCounter(double averagingPeriod, double averagingWeight)
		{
			this.averagingPeriod = averagingPeriod;
			this.averagingWeight = averagingWeight;
		}

		/// <summary>Call this to record a new frame.</summary>
		public void NewFrame()
		{
			frameCounter++;
			if (DateTime.Now > lastTime.AddSeconds(averagingPeriod))
			{
				if (FPS == 0)
					FPS = (frameCounter / (DateTime.Now - lastTime).TotalSeconds);
				else
					FPS = averagingWeight * (frameCounter / (DateTime.Now - lastTime).TotalSeconds) + (1 - averagingWeight) * FPS;

				// reset counter
				frameCounter = 0;
				lastTime = DateTime.Now;
			}
		}

		//protected Queue<DateTime> frameTimes;
		//protected double averagingPeriod;

		//public FrameRateCounter(double averagingPeriod)
		//{
		//    this.frameTimes = new Queue<DateTime>();
		//    this.averagingPeriod = averagingPeriod;
		//}

		//public void NewFrame()
		//{
		//    DateTime now = DateTime.Now;
		//    frameTimes.Enqueue(DateTime.Now);
		//    while (frameTimes.Peek() < now.AddSeconds(-averagingPeriod))
		//        frameTimes.Dequeue();
		//}

		//public double FPS
		//{
		//    get
		//    {
		//        if (frameTimes.Count == 0)
		//            return 0;
		//        return 1.0e7f * frameTimes.Count() / (DateTime.Now.Ticks - frameTimes.Peek().Ticks);
		//    }
		//}
	}
}
